package com.xzzn.ems.mapper;

import java.util.List;
import java.util.Map;

import com.xzzn.ems.domain.EmsDevicesSetting;

/**
 * Modbus设备配置Mapper接口
 *
 * @author xzzn
 * @date 2025-06-24
 */
public interface EmsDevicesSettingMapper
{
    /**
     * 查询Modbus设备配置
     *
     * @param id Modbus设备配置主键
     * @return Modbus设备配置
     */
    public EmsDevicesSetting selectEmsDevicesSettingById(Long id);

    /**
     * 查询Modbus设备配置列表
     *
     * @param emsDevicesSetting Modbus设备配置
     * @return Modbus设备配置集合
     */
    public List<EmsDevicesSetting> selectEmsDevicesSettingList(EmsDevicesSetting emsDevicesSetting);

    /**
     * 新增Modbus设备配置
     *
     * @param emsDevicesSetting Modbus设备配置
     * @return 结果
     */
    public int insertEmsDevicesSetting(EmsDevicesSetting emsDevicesSetting);

    /**
     * 修改Modbus设备配置
     *
     * @param emsDevicesSetting Modbus设备配置
     * @return 结果
     */
    public int updateEmsDevicesSetting(EmsDevicesSetting emsDevicesSetting);

    /**
     * 删除Modbus设备配置
     *
     * @param id Modbus设备配置主键
     * @return 结果
     */
    public int deleteEmsDevicesSettingById(Long id);

    /**
     * 批量删除Modbus设备配置
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEmsDevicesSettingByIds(Long[] ids);

    /**
     * 根据site_id获取所有电池堆
     * @param siteId
     * @return
     */
    public List<Map<String, Object>> getAllStackInfoBySiteId(Long siteId);

    /**
     * 根据电池堆id获取所有电池簇
     * @param stackDeviceId
     * @return
     */
    public List<Map<String, Object>> getAllClusterInfoByStackId(Long stackDeviceId);

    /**
     * 获取设备通信状态
     * @param deviceId
     * @return
     */
    public Map<String, Object> getDeviceStatus(Long deviceId);
}
