package com.xzzn.ems.service.impl;

import java.util.List;
import com.xzzn.common.utils.DateUtils;
import com.xzzn.ems.domain.EmsTicket;
import com.xzzn.ems.mapper.EmsTicketMapper;
import com.xzzn.ems.service.IEmsTicketService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


/**
 * 工单主Service业务层处理
 * 
 * @author xzzn
 * @date 2025-06-26
 */
@Service
public class EmsTicketServiceImpl implements IEmsTicketService
{
    @Autowired
    private EmsTicketMapper emsTicketMapper;

    /**
     * 查询工单主
     * 
     * @param id 工单主主键
     * @return 工单主
     */
    @Override
    public EmsTicket selectEmsTicketById(String id)
    {
        return emsTicketMapper.selectEmsTicketById(id);
    }

    /**
     * 查询工单主列表
     * 
     * @param emsTicket 工单主
     * @return 工单主
     */
    @Override
    public List<EmsTicket> selectEmsTicketList(EmsTicket emsTicket)
    {
        return emsTicketMapper.selectEmsTicketList(emsTicket);
    }

    /**
     * 新增工单主
     * 
     * @param emsTicket 工单主
     * @return 结果
     */
    @Override
    public int insertEmsTicket(EmsTicket emsTicket)
    {
        emsTicket.setCreateTime(DateUtils.getNowDate());
        return emsTicketMapper.insertEmsTicket(emsTicket);
    }

    /**
     * 修改工单主
     * 
     * @param emsTicket 工单主
     * @return 结果
     */
    @Override
    public int updateEmsTicket(EmsTicket emsTicket)
    {
        emsTicket.setUpdateTime(DateUtils.getNowDate());
        return emsTicketMapper.updateEmsTicket(emsTicket);
    }

    /**
     * 批量删除工单主
     * 
     * @param ids 需要删除的工单主主键
     * @return 结果
     */
    @Override
    public int deleteEmsTicketByIds(String[] ids)
    {
        return emsTicketMapper.deleteEmsTicketByIds(ids);
    }

    /**
     * 删除工单主信息
     * 
     * @param id 工单主主键
     * @return 结果
     */
    @Override
    public int deleteEmsTicketById(String id)
    {
        return emsTicketMapper.deleteEmsTicketById(id);
    }
}
