package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.EmsBatteryData;
import com.xzzn.ems.domain.EmsCoolingData;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.*;
import com.xzzn.ems.service.ISingleSiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 单点监控 服务层实现
 */
@Service
public class SingleSiteServiceImpl implements ISingleSiteService {

    private static final String CLUSTER_DATA_TEP = "温度";

    private static final String CLUSTER_DATA_VOLTAGE = "电压";

    private static final String CLUSTER_DATA_SOC = "SOC";

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsAlarmRecordsMapper emsAlarmRecordsMapper;
    @Autowired
    private EmsBatteryDataMapper emsBatteryDataMapper;
    @Autowired
    private EmsBatteryStackMapper emsBatteryStackMapper;
    @Autowired
    private EmsBatteryClusterMapper emsBatteryClusterMapper;
    @Autowired
    private EmsPcsBranchDataMapper emsPcsBranchDataMapper;
    @Autowired
    private EmsCoolingDataMapper emsCoolingDataMapper;
    @Autowired
    private EmsAmmeterDataMapper emsAmmeterDataMapper;
    @Autowired
    private EmsDevicesSettingMapper emsDevicesSettingMapper;

    @Override
    public SiteMonitorHomeVo getSiteMonitorDataVo(Long siteId) {
        SiteMonitorHomeVo siteMonitorHomeVo = new SiteMonitorHomeVo();

        if (siteId != null) {
            // 实时告警数据 名称+状态+告警内容
            List<SiteMonitorHomeAlarmVo> siteMonitorHomeAlarmVo = emsAlarmRecordsMapper.getAlarmRecordsBySiteId(siteId);
            siteMonitorHomeVo.setSiteMonitorHomeAlarmVo(siteMonitorHomeAlarmVo);
            // 能量数据
            List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId);
            if (!CollectionUtils.isEmpty(siteMonitorDataVoList)) {
                BigDecimal dayChargeCap = new BigDecimal(0);
                BigDecimal dayDisChargeCap = new BigDecimal(0);
                BigDecimal totalChargeCap = new BigDecimal(0);
                BigDecimal totalDisChargeCap = new BigDecimal(0);
                for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                    // 总充电量
                    totalChargeCap = totalChargeCap.add(sitePcsData.getChargedCap());
                    // 总放电量
                    totalDisChargeCap = totalDisChargeCap.add(sitePcsData.getDisChargedCap());
                    // 获取当天的充电量+放电量
                    String pcsDate = sitePcsData.getAmmeterDate();
                    boolean isToday= checkIsToday(pcsDate);
                    if(isToday){
                        dayChargeCap = dayChargeCap.add(sitePcsData.getChargedCap());
                        dayDisChargeCap = dayDisChargeCap.add(sitePcsData.getDisChargedCap());
                    }
                }
                siteMonitorHomeVo.setDayChargedCap(dayChargeCap);
                siteMonitorHomeVo.setDayDisChargedCap(dayDisChargeCap);
                siteMonitorHomeVo.setTotalChargedCap(totalChargeCap);
                siteMonitorHomeVo.setTotalDischargedCap(totalDisChargeCap);
                // 储能可用电量
                BigDecimal energyStorageAvailElec = siteMonitorHomeVo.getTotalDischargedCap().subtract(siteMonitorHomeVo.getTotalChargedCap());
                siteMonitorHomeVo.setEnergyStorageAvailElec(energyStorageAvailElec);
            }
            siteMonitorHomeVo.setSiteMonitorDataVo(siteMonitorDataVoList);
            // 电网实时功率
            siteMonitorHomeVo.setGridNrtPower(emsPcsDataMapper.getGridNrtPower(siteId));
        }

        return siteMonitorHomeVo;
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    // 获取单站监控实时运行头部数据
    @Override
    public SiteMonitorRuningHeadInfoVo getSiteRunningHeadInfo(Long siteId) {
        SiteMonitorRuningHeadInfoVo siteMonitorRunningHeadInfoVo = new SiteMonitorRuningHeadInfoVo();

        if (siteId != null) {
            // 实时有功功率/实时无功功率/今日充电量/今日放电量
            siteMonitorRunningHeadInfoVo = emsPcsDataMapper.getSiteRunningHeadInfo(siteId);
            // 电池簇SOC
            // 电池堆SOH
            EmsBatteryData emsBatteryData = emsBatteryDataMapper.getBatteryDataBySiteId(siteId);
            if (emsBatteryData != null) {
                siteMonitorRunningHeadInfoVo.setSoc(emsBatteryData.getSoc());
                siteMonitorRunningHeadInfoVo.setSoh(emsBatteryData.getSoh());
            }
        }

        return siteMonitorRunningHeadInfoVo;
    }

    //  获取单站监控实时运行曲线图数据
    @Override
    public SiteMonitorRuningInfoVo getRunningGraph(Long siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (siteId != null) {
            //储能功率list
            //pcs平均温度list
            //电池平均soclist
            //电池平均温度list
        }
        return siteMonitorRuningInfoVo;
    }

    // 根据site_id获取pcs详细数据+支路数据
    @Override
    public List<PcsDetailInfoVo> getPcsDetailInfo(Long siteId) {
        List<PcsDetailInfoVo> pcsDetailInfoVoList = new ArrayList<>();

        if (siteId != null) {
            // 获取pcs最新数据
            pcsDetailInfoVoList = emsPcsDataMapper.getPcsDetailInfoBySiteId(siteId);
            if (!CollectionUtils.isEmpty(pcsDetailInfoVoList)) {
                for (PcsDetailInfoVo pcsDetailInfoVo : pcsDetailInfoVoList) {
                    Long deviceId = pcsDetailInfoVo.getDeviceId();
                    // 获取支路最新数据
                    if (deviceId != null) {
                        List<PcsBranchInfo> pcsBranchInfoList = emsPcsBranchDataMapper.getPcsBranchInfoList(siteId, deviceId);
                        pcsDetailInfoVo.setPcsBranchInfoList(pcsBranchInfoList);
                    }
                }
            }
        }
        return pcsDetailInfoVoList;
    }

    // 获取BMS总览数据
    @Override
    public List<BMSOverViewVo> getBMSOverView(Long siteId) {
        List<BMSOverViewVo> bmsOverViewVoList = new ArrayList<>();

        if (siteId != null) {
            // 获取电池堆list
            bmsOverViewVoList  = emsBatteryStackMapper.selectEmsBatteryStackBySiteId(siteId);
            if (!CollectionUtils.isEmpty(bmsOverViewVoList)) {
                for (BMSOverViewVo bmsOverViewVo : bmsOverViewVoList) {
                    // 获取单体电池数据-待确认
                    Long stackDeviceId = bmsOverViewVo.getDeviceId();
                    if (stackDeviceId != null) {
                        List<BMSBatteryDataList> batteryDataList = new ArrayList<>();
                        batteryDataList = emsBatteryClusterMapper.getBmsBatteryData(siteId,stackDeviceId);
                        bmsOverViewVo.setBatteryDataList(batteryDataList);
                    }
                }
            }
        }
        return bmsOverViewVoList;
    }

    // 获取BMS电池簇数据
    @Override
    public List<BMSBatteryClusterVo> getBMSBatteryCluster(Long siteId) {
        List<BMSBatteryClusterVo> bmsBatteryClusterVoList = new ArrayList<>();

        if (siteId != null) {
            bmsBatteryClusterVoList = emsBatteryClusterMapper.getBMSBatteryCluster(siteId);
            if (!CollectionUtils.isEmpty(bmsBatteryClusterVoList)) {
                for (BMSBatteryClusterVo bmsBatteryClusterVo : bmsBatteryClusterVoList) {

                    Long clusterDeviceId = bmsBatteryClusterVo.getDeviceId();
                    List<BMSBatteryClusterDataList> clusterDataList = new ArrayList<>();
                    if (clusterDeviceId != null) {
                        // 获取单体电池数据-平均/最大/最小
                        BatteryClusterDataDetailVo  batteryClusterDataDetailVo = emsBatteryDataMapper.getBatteryDataByClusterId(siteId,clusterDeviceId);
                        // 处理数据
                        if (batteryClusterDataDetailVo != null) {
                            BMSBatteryClusterDataList voltageData = new BMSBatteryClusterDataList();
                            BMSBatteryClusterDataList tempData = new BMSBatteryClusterDataList();
                            BMSBatteryClusterDataList socData = new BMSBatteryClusterDataList();
                            // 设值
                            voltageData.setDataName(CLUSTER_DATA_VOLTAGE);
                            voltageData.setAvgData(batteryClusterDataDetailVo.getAvgVoltage());
                            voltageData.setMaxData(batteryClusterDataDetailVo.getMaxVoltage());
                            voltageData.setMinData(batteryClusterDataDetailVo.getMinVoltage());
                            tempData.setDataName(CLUSTER_DATA_TEP);
                            tempData.setAvgData(batteryClusterDataDetailVo.getAvgTemp());
                            tempData.setMaxData(batteryClusterDataDetailVo.getMaxTemp());
                            tempData.setMinData(batteryClusterDataDetailVo.getMinTemp());
                            socData.setDataName(CLUSTER_DATA_SOC);
                            socData.setAvgData(batteryClusterDataDetailVo.getAvgSoc());
                            socData.setMaxData(batteryClusterDataDetailVo.getMaxSoc());
                            socData.setMinData(batteryClusterDataDetailVo.getMinSoc());

                            // 设置对应单体id
                            List<Map<String,Object>> dataIdMapList = emsBatteryDataMapper.getDataIdsMap(batteryClusterDataDetailVo);
                            Map<String,Long> resultIdMap = new HashMap<>();
                            if (!CollectionUtils.isEmpty(dataIdMapList)) {
                                resultIdMap = dataIdMapList.stream()
                                        .filter(map -> map.containsKey("type") && map.containsKey("device_id")) // 过滤无效数据
                                        .filter(map -> map.get("type") instanceof String && map.get("device_id") instanceof Number) // 确保类型正确
                                        .collect(Collectors.toMap(
                                                map -> (String) map.get("type"), // 键：type 字段
                                                map -> ((Number) map.get("device_id")).longValue(), // 值：device_id 转为 int
                                                (existing, replacement) -> existing // 处理键冲突的策略
                                        ));

                                // 电压
                                voltageData.setMaxDataID(resultIdMap.get("maxVoltageId"));
                                voltageData.setMaxDataID(resultIdMap.get("maxVoltageId"));
                                // 温度
                                tempData.setMaxDataID(resultIdMap.get("maxTempId"));
                                tempData.setMinDataID(resultIdMap.get("minTempId"));
                                // soc
                                socData.setMaxDataID(resultIdMap.get("maxSocId"));
                                socData.setMinDataID(resultIdMap.get("minSocId"));
                            }
                            clusterDataList.add(voltageData);
                            clusterDataList.add(tempData);
                            clusterDataList.add(socData);
                        }
                        bmsBatteryClusterVo.setBatteryDataList(clusterDataList);
                    }
                }
            }
        }
        return bmsBatteryClusterVoList;
    }

    // 获取液冷设备参数
    @Override
    public  List<EmsCoolingData> getCoolingDataList(Long siteId) {
        List<EmsCoolingData> emsCoolingDataList = new ArrayList<>();
        if (siteId != null) {
            emsCoolingDataList = emsCoolingDataMapper.getCoolingDataList(siteId);
        }
        return emsCoolingDataList;
    }


    /**
     * 根据电池堆获取单体柱状数据
     * @param clusterDeviceId
     * @return
     */
    @Override
    public BatteryDataStatsListVo getClusterDataInfoList(Long clusterDeviceId) {
        return null;
    }

    /**
     * 电表数据信息
     * @param siteId
     * @return
     */
    @Override
    public List<AmmeterDataVo> getAmmeterDataList(Long siteId) {
        List<AmmeterDataVo> ammeterDataVos = new ArrayList<>();
        AmmeterDataVo ammeterDataVo = new AmmeterDataVo();
        if (siteId != null) {
            List<AmmeterDataDetailInfo> ammeterDataDetailInfos = emsAmmeterDataMapper.getAmmeterDetailInfo(siteId);
            ammeterDataVo.setAmmeterDataDetailInfos(ammeterDataDetailInfos);
            if (!CollectionUtils.isEmpty(ammeterDataDetailInfos)) {
                // 获取通信状态
                Long deviceId = ammeterDataDetailInfos.get(0).getDeviceId();
                Map<String,Object> map = emsDevicesSettingMapper.getDeviceStatus(deviceId);
                ammeterDataVo.setEmsCommunicationStatus(map.get("communication_status").toString());
                // 数据更新时间
                ammeterDataVo.setDataUpdateTime(ammeterDataDetailInfos.get(0).getUpdateTime());
            }
        }
        ammeterDataVos.add(ammeterDataVo);
        return ammeterDataVos;
    }

}
